@@ -1,8 +1,8 @@
 /*
-    $Id: _cdio_bincue.c,v 1.37 2003/11/04 04:44:43 rocky Exp $
+    $Id: _cdio_bincue.c,v 1.41 2004/02/08 03:35:24 rocky Exp $
 
     Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
-    Copyright (C) 2002,2003 Rocky Bernstein <rocky@panix.com>
+    Copyright (C) 2002, 2003, 2004 Rocky Bernstein <rocky@panix.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
    (*.cue).
 */
 
-static const char _rcsid[] = "$Id: _cdio_bincue.c,v 1.37 2003/11/04 04:44:43 rocky Exp $";
+static const char _rcsid[] = "$Id: _cdio_bincue.c,v 1.41 2004/02/08 03:35:24 rocky Exp $";
 
 #include "cdio_assert.h"
 #include "cdio_private.h"
@@ -371,8 +371,7 @@ _cdio_image_read_cue (_img_private_t *_obj)
       }
 
     } else if (2==sscanf(p, "TRACK %d MODE1/%d", &track_num, &blocksize)) {
-      track_info_t  *this_track=&(_obj->tocent[_obj->total_tracks]);
-      this_track->blocksize      = blocksize;
+      track_info_t *this_track=&(_obj->tocent[_obj->total_tracks]);
       this_track->blocksize   = blocksize;
       switch(blocksize) {
       case 2048:
@@ -576,12 +575,14 @@ _cdio_read_mode2_sectors (void *env, void *data, uint32_t lsn,
   if (NULL != obj) free(obj);
 
 static void 
-_cdio_bincue_free (void *env) {
-  _img_private_t *_obj = env;
+_cdio_bincue_destroy (void *obj) 
+{
+  _img_private_t *env = obj;
 
-  if (NULL == _obj) return;
-  free_if_notnull(_obj->mcn);
-  cdio_generic_stream_free(_obj);
+  if (NULL == env) return;
+  free_if_notnull(env->mcn);
+  free(env->cue_name);
+  cdio_generic_stdio_free(env);
 }
 
 /*!
@@ -591,9 +592,6 @@ _cdio_bincue_free (void *env) {
 
   0 is returned if no error was found, and nonzero if there as an error.
 */
-/*!
-  Set the device to use in I/O operations.
-*/
 static int
 _cdio_set_arg (void *env, const char key[], const char value[])
 {
@@ -885,7 +883,7 @@ cdio_open_common (_img_private_t **_data)
 {
   cdio_funcs _funcs = {
     .eject_media        = cdio_generic_bogus_eject_media,
-    .free               = _cdio_bincue_free,
+    .free               = _cdio_bincue_destroy,
     .get_arg            = _cdio_get_arg,
     .get_default_device = cdio_get_default_device_bincue,
     .get_first_track_num= _cdio_get_first_track_num,
@@ -918,6 +916,7 @@ cdio_open_bincue (const char *source_name)
   char *bin_name = cdio_is_cuefile(source_name);
 
   if (NULL != bin_name) {
+    free(bin_name);
     return cdio_open_cue(source_name);
   } else {
     char *cue_name = cdio_is_binfile(source_name);
@@ -951,7 +950,7 @@ cdio_open_cue (const char *cue_name)
   if (_cdio_init(_data)) {
     return ret;
   } else {
-    cdio_generic_stream_free (_data);
+    cdio_generic_stdio_free (_data);
     return NULL;
   }
 }
