@@ -1,10 +1,7 @@
 #!/bin/sh
-#$Id: check_cue.sh.in,v 1.8 2003/09/01 02:02:59 rocky Exp $
-if test -z "-L/usr/local/lib -lcddb  " ; then
-  cddb_opt='--no-cddb'
-fi
-
-if test -z "-L/usr/local/lib -lvcdinfo -lvcd -liso9660 -lcdio  " ; then
+#$Id: check_cue.sh.in,v 1.15 2003/10/06 04:04:05 rocky Exp $
+# Tests to see that BIN/CUE file iamge reading is correct (via cd-info). 
+if test -n "-L/usr/local/lib -lvcdinfo -lvcd -liso9660 -lcdio  " ; then
   vcd_opt='--no-vcd'
 fi
 
@@ -14,7 +11,7 @@ fi
 
 . ${srcdir}/check_common_fn
 
-if ! test -x ../src/cd-info ; then
+if test ! -x ../src/cd-info ; then
   exit 77
 fi
 
@@ -22,34 +19,39 @@ BASE=`basename $0 .sh`
 
 fname=cdda
 testnum=CD-DA
-test_cdinfo "--cue-file ${srcdir}/${fname}.cue $cddb_opt" \
+test_cdinfo "--quiet --cue-file ${srcdir}/${fname}.cue --no-cddb" \
   ${fname}.dump ${srcdir}/${fname}.right
 RC=$?
-check_result $RC "cdinfo CUE test $testnum"
+check_result $RC "cd-info CUE test $testnum"
 
-test_cdinfo "--bin-file ${srcdir}/${fname}.bin $cddb_opt" \
+test_cdinfo "--quiet --bin-file ${srcdir}/${fname}.bin --no-cddb" \
   ${fname}.dump ${srcdir}/${fname}.right
 RC=$?
-check_result $RC "cdinfo BIN test $testnum"
+check_result $RC "cd-info BIN test $testnum"
 
 fname=isofs-m1
 testnum='ISO 9660 mode1'
 if test -f  ${srcdir}/${fname}.bin ; then
-  test_cdinfo "--cue-file ${srcdir}/${fname}.cue --iso9660 " \
+  test_cdinfo "-q --cue-file ${srcdir}/${fname}.cue --iso9660" \
     ${fname}.dump ${srcdir}/${fname}.right
   RC=$?
-  check_result $RC "cdinfo CUE test $testnum"
+  check_result $RC "cd-info CUE test $testnum"
 else 
   echo "Don't see CUE file ${srcdir}/${fname}.bin. Test $testum skipped."
 fi
 
 fname=vcd_demo
+if test -z "-L/usr/local/lib -lvcdinfo -lvcd -liso9660 -lcdio  " ; then
+  right=${srcdir}/${fname}.right
+else
+  right=${srcdir}/${fname}_vcdinfo.right
+fi
 testnum='Video CD'
 if test -f ${srcdir}/${fname}.cue ; then
-  test_cdinfo "-c ${srcdir}/vcd_demo.cue $vcd_opt --iso9660" \
-      ${fname}.dump ${srcdir}/${fname}.right
+  test_cdinfo "-q -c ${srcdir}/vcd_demo.cue --iso9660" \
+      ${fname}.dump $right
   RC=$?
-  check_result $RC "cdinfo CUE test $testnum"
+  check_result $RC "cd-info CUE test $testnum"
 else 
   echo "Don't see CUE file ${srcdir}/${fname}.cue. Test $testum skipped."
 fi
@@ -57,10 +59,10 @@ fi
 fname=svcd_ogt_test_ntsc
 testnum='Super Video CD'
 if test -f ${srcdir}/${fname}.bin ; then
-  test_cdinfo "--cue-file ${srcdir}/${fname}.cue $vcd_opt --iso9660 " \
+  test_cdinfo "-q --cue-file ${srcdir}/${fname}.cue $vcd_opt --iso9660" \
     ${fname}.dump ${srcdir}/${fname}.right
   RC=$?
-  check_result $RC "cdinfo CUE test $testnum"
+  check_result $RC "cd-info CUE test $testnum"
 else 
   echo "Don't see CUE file ${srcdir}/${fname}.bin. Test $testnum skipped."
 fi
