@@ -1,7 +1,7 @@
 #!/bin/sh
-#$Id: check_nrg.sh.in,v 1.6 2003/09/01 02:02:59 rocky Exp $
+#$Id: check_nrg.sh.in,v 1.9 2003/10/06 04:04:05 rocky Exp $
 
-if test -z "-L/usr/local/lib -lvcdinfo -lvcd -liso9660 -lcdio  " ; then
+if test -n "-L/usr/local/lib -lvcdinfo -lvcd -liso9660 -lcdio  " ; then
   vcd_opt='--no-vcd'
 fi
 
@@ -11,14 +11,14 @@ fi
 
 . ${srcdir}/check_common_fn
 
-if ! test -x ../src/cd-info ; then
+if test ! -x ../src/cd-info ; then
   exit 77
 fi
 
 BASE=`basename $0 .sh`
 fname=videocd
 
-test_cdinfo "--nrg-file ${srcdir}/${fname}.nrg $vcd_opt --iso9660 " \
+test_cdinfo "--quiet --nrg-file ${srcdir}/${fname}.nrg $vcd_opt --iso9660 " \
   ${fname}.dump ${srcdir}/${fname}.right
 RC=$?
 check_result $RC 'cd-info NRG test 1'
@@ -27,7 +27,7 @@ BASE=`basename $0 .sh`
 nrg_file=${srcdir}/monvoisin.nrg
 
 if test -f  $nrg_file ; then
-  test_cdinfo "--nrg-file $nrg_file $vcd_opt --iso9660 " \
+  test_cdinfo "-q --nrg-file $nrg_file $vcd_opt --iso9660 " \
     monvoisin.dump ${srcdir}/monvoisin.right
   RC=$?
   check_result $RC 'cd-info NRG test 2'
@@ -38,7 +38,7 @@ fi
 
 nrg_file=${srcdir}/svcdgs.nrg
 if test -f  $nrg_file ; then
-  test_cdinfo "--nrg-file $nrg_file $vcd_opt --iso9660" \
+  test_cdinfo "-q --nrg-file $nrg_file $vcd_opt --iso9660" \
     svcdgs.dump ${srcdir}/svcdgs.right
   RC=$?
   check_result $RC 'cd-info NRG test 3'
